==> MuSE-0.5.tar.gz: MuSE-0.5/jmixer.cpp <==
/* jmixer.cpp - part of MUSE
 * (c)2000 Denis Roio aka jaromil
 * c++ engine for multiple mp3 realtime stream mixing
 */

#include <iostream.h>
#include <stdio.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/soundcard.h>
#include <string.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <termios.h>
#include <sys/time.h>
#include <config.h>

#include "jmixer.h"
#include "audioproc.h"

Stream_mixer::Stream_mixer() {

  /* well... let's initialize channels with mp3 decoders */
  int i;
  for(i=0;i<MAX_CHANNELS;i++)
    xach[i] = new XaudioChannel();
  
  dsp = 0;
  have_gui = false;

  dspout = false;
  icecastout = false;
  fileout = false;
  quit = false;

  /* Initialization as a synchronous object */
  synchronous.registerObjectName("Stream_mixer");

}

void Stream_mixer::run() {

  if(debug)
    cerr << selfName() << " running on thread " << (int)pthread_self() << "\n";

  while(!quit) {
    
    synchronous.select.accept(0, id_stream_mixer);

    synchronous.select.waitUntil(1, LTI::Kernel::now()+50);
    synchronous.wait();

    cafudda();
  }

  cerr << selfName() << " exiting " << "\n";

}

Stream_mixer::~Stream_mixer() {
  int i;

  if(dsp!=0)
    close_soundcard();
  
  if(icecastout)
    stop_icecast();

  for(i=0;i<MAX_CHANNELS;i++)
    delete xach[i];

}

void Stream_mixer::register_gui(Amuse *reg_gui) { 
  gui = reg_gui; 
  have_gui = true;
  if(HAVE_LAME) {
    char temp[256];
    lame.version();
    sprintf(temp,"Encoding with Lame %s",lame.ver);
    gui->status_lame->value(temp);
  }
  int c;
  for(c=0;c<MAX_CHANNELS;c++) {
    if(playlist[c].loaded) {
      int cc;
      char *p;
      for(cc=0;cc<playlist[c].length;cc++) {
	p = playlist[c].song(cc);
	while(*p!='\0') p++;
	while(*p!='/') p--;
	gui->chan[c]->playlist_browser->add(p+1,NULL);
      }
    }
  }
}

bool Stream_mixer::open_soundcard()
{
    int format,tstereo,speed,caps;
    static int val;
    
    if((dsp=open("/dev/dsp",O_RDWR))==-1) {
        perror("Error opening /dev/dsp: ");
	return(false);
    } else
      fprintf(stderr,"Found soundcard on /dev/dsp\n");

    /* BUFFER FRAGMENTATION */
    val = (FRAGCOUNT << 16) | FRAGSIZE;

    if(ioctl(dsp,SNDCTL_DSP_SETFRAGMENT,&val)==-1)
      fprintf(stderr,"failed to set dsp buffers\n");

    /* or LE */
    format = AFMT_S16_NE;

    tstereo = 1; /* only stereo _DSP_ in/out */
    speed = SAMPLE_RATE; /* 44100hz mixing */

    ioctl(dsp, SNDCTL_DSP_GETCAPS, &caps);
    if(caps & DSP_CAP_DUPLEX) {
      fullduplex = true;
      fprintf(stderr,"full duplex supported. good\n");
      ioctl(dsp, SNDCTL_DSP_SETDUPLEX, 0);
      ioctl(dsp, DSP_CAP_DUPLEX, 0);
    } else {
      fprintf(stderr,"full duplex is not supported\n");
      fullduplex = false;
    }

    /* FORMAT */
    if(ioctl(dsp,SNDCTL_DSP_SETFMT,&format)==-1)
      fprintf(stderr,"failed to set data format\n");

    /* CHANNELS */
    if(ioctl(dsp,SNDCTL_DSP_STEREO,&tstereo)==-1)
      fprintf(stderr,"something went wrong with the stereo setting\n");

    /* SAMPLERATE */
    if(ioctl(dsp,SNDCTL_DSP_SPEED,&speed)==-1)
      fprintf(stderr,"speed setting failed\n");

    /* GET FRAG SIZE BACK */
    if(ioctl(dsp,SNDCTL_DSP_GETBLKSIZE,&val)==-1)
      fprintf(stderr,"get block size failed to return\n");

    fprintf(stderr,"mixing 16bit %dHz stereo\n",speed);

    dspout = false;

    livein = new LiveIn(speed, tstereo+1, &dsp);

    return(true);
} /* open_soundcard */

bool Stream_mixer::set_icecast(char *ip, int port, char *pass, char *mountpoint, int bps, char *name, char *url, char *desc, int login) {

  if(!ice.set_server(ip,port,pass,mountpoint,bps,login)) {
    if(have_gui)
      gui->status_main->value("doom damnit! i can't resolve icecast hostname");
    else
      fprintf(stderr,"Can't resolve icecast server hostname\n");
    synchronous.reply();
    return(false);
  }

  ice.set_meta(name, url, desc);
      
  if((!lame.initialized)||(lame.bps!=bps)) {
    gui->status_main->value("ouch! lame is not initialized!");
    synchronous.reply();
    return(false);
  }
  
  if(ice.start()) { /* we are connected */
    icecastout = true;

    /* notice the user */
    if(have_gui) {
      char temp[256];
      gui->status_main->value("WE GOT MILK !!");
      sprintf(temp,"STREAMING ON [ http://%s:%u/%s ]",ip,port,mountpoint);
      gui->status_ice->value(temp);
    } else
      fprintf(stderr,"Estabilished %ubps stream on %s:%u/%s\n",bps,ip,port,mountpoint);

  } else { /* connection failed */
    
    icecastout = false;
    if(have_gui) {
      char temp[256];
      sprintf(temp,"Got problems streaming on server %s. No milk??",ip);
      gui->status_main->value(temp);
    } else
      fprintf(stderr,"Got problems streaming on server %s. No milk??\n",ip);
  }
  
  synchronous.reply();
  return(icecastout);
}

void Stream_mixer::set_lame(int bps, int freq, int mode, int quality, int lowpass, int highpass) {
  lame.init(bps, freq, mode, quality, lowpass, highpass);
  synchronous.reply();
}

void Stream_mixer::stop_icecast() {
  ice.stop();
  icecastout = false;

  if (have_gui) {
    gui->status_main->value("Stopped streaming to icecast server");
    gui->status_ice->value("");
  } else
    fprintf(stderr,"Stopped streaming to icecast server\n");

  synchronous.reply();
}

int Stream_mixer::set_mp3file(char *file, int bps) {
  int res;
  
  res = mp3file.openmp3(file);
  if(res>0) fileout = true;

  /*  
      if((!lame.initialized)||(lame.bps!=bps))
      lame.init(bps);
  */
  
  synchronous.reply();
  return(res);
}

void Stream_mixer::stop_mp3file() {
  mp3file.closemp3();
  fileout = false;
  synchronous.reply();
}

void Stream_mixer::cafudda()
{
  int i,j;
  int num = 0;
  float res;
  int max = 0;

  /* j is the number of 16bit samples (not counting stereo inrterleaving)*/
  j=BUF_SIZE>>1;
  
  memset(process_buffer,0,BUF_SIZE<<2); /* assuming sizeof(int)==4 */

  have_sound = false;

  for(i=0;i<MAX_CHANNELS;i++) {

    if(xach[i]->on) {
      res = xach[i]->mix(process_buffer);
      updchan(i,res);
      if(max < xach[i]->in_smp) max = xach[i]->in_smp;

      /* notice that we have pcm to process (at least 1 chan) */
      have_sound = true;
      
    } /* if(xach[i].on) */
  } /* for(i=0;i<MAX_CHANNELS;i++) */
  
  if(livein->on) {
    int ires;
    ires = livein->mix(process_buffer);
    if(max < ires) max = ires;
    
    /* notice that we have pcm to process (at least 1 chan) */
    have_sound = true;
  }
  
  if(have_sound) {
    
    /* CLIPPING */
    clip_audio(process_buffer,audio_buffer,max);
    
    /* ENCODE AUDIO */
    if(fileout||icecastout)
      lame.encode(&audio_buffer[0],max>>2);
    
    /* WRITE 2 MP3 FILE */
    if(fileout)
      mp3file.writemp3();
    
    /* SEND 2 ICECAST */
    if(icecastout) {
      if(ice.send()!=0) {
	if(have_gui)
	  gui->status_main->value("Send error: cannot send stream to icecast server");
	else
	  fprintf(stderr,"Send error: cannot send stream to icecast server\n");
      }
    }

    /* WRITE 2 DSP */
    if(dspout)
      write(dsp,audio_buffer,max);

  }

  /* compute and draw levels */
  if(have_gui)
    gui->mix_monitor->draw_levels(get_level(audio_buffer,max));
     
}

void Stream_mixer::pause_channel(int ch) {

  if(xach[ch]->opened)
    xach[ch]->on = !xach[ch]->on;
  
  synchronous.reply();
}

bool Stream_mixer::set_channel(int ch, int playlist_pos) {

  bool res;
  char *sel = playlist[ch].song(playlist_pos);

  if(sel==NULL)
    return(false);


  if(strstr(sel,".ogg")!=NULL) {
#ifdef HAVE_VORBIS
    if(xach[ch]->type != OGGCHAN) {
      delete xach[ch];
      xach[ch] = new OggChannel();
    }
#else
    fprintf(stderr,"can't open .ogg files, OggVorbis support not compiled in\n");
    synchronous.reply();
    return(false);
#endif
  } else if(strstr(sel,".mp3")!=NULL) {
    if(xach[ch]->type != MP3CHAN) {
      delete xach[ch];
      xach[ch] = new XaudioChannel();
    }
  }
  
  res = xach[ch]->set(playlist[ch].song(playlist_pos));

  if(!res) {
    if(have_gui) {
      gui->status_main->value("Doom damnit! can't open the file selected");
    } else {
      fprintf(stderr,"error opening file %s\n", playlist[ch].song(playlist_pos));
    }
  } else {
    playlist[ch].select(playlist_pos);
    if(have_gui) {
      char temp[30];
      sprintf(temp,"File loaded on channel [%u]",ch);
      gui->status_main->value(temp);
    }
  }

  synchronous.reply();
  return(res);
}

void Stream_mixer::set_all_volumes(float *vol) {
  int c;
  for(c=0;c<MAX_CHANNELS;c++) {
    xach[c]->volume = vol[c];
  }
  synchronous.reply();
}

void Stream_mixer::set_volume(int ch, float vol) {

  xach[ch]->volume = vol;

  synchronous.reply();
}

void Stream_mixer::set_speed(int ch, int speed) {

  xach[ch]->speed = speed;

  synchronous.reply();
}

void Stream_mixer::play_channel(int ch) {
  
  if(xach[ch]->opened) {
    xach[ch]->play();
  }

  synchronous.reply();
}
  
bool Stream_mixer::stop_channel(int ch) {

  bool res = true;

  if(ch<=MAX_CHANNELS) {
    res = xach[ch]->stop();
  }

  synchronous.reply();
  return(res);

}

bool Stream_mixer::set_position(float pos, int ch) {
  bool res = true;

  res = xach[ch]->pos(pos);

  synchronous.reply();
  return(res);
}

bool Stream_mixer::set_live(bool stat) {
  if(!((dspout)&&(!fullduplex)&&(stat)))
    livein->on = stat;

  synchronous.reply();
  return(livein->on&stat);
}

bool Stream_mixer::set_lineout(bool stat) {
  if(!((livein->on)&&(!fullduplex)&&(stat)))
    dspout = stat;

  synchronous.reply();
  return(dspout&stat);
}

void Stream_mixer::close_soundcard() {
  ioctl(dsp, SNDCTL_DSP_RESET, 0);
  close(dsp);
}

bool Stream_mixer::set_playlist(char *file, int ch) {
  bool res = false;
  res = playlist[ch].set(file);
  if(res) {
    if(have_gui) {
      gui->chan[ch]->playlist_browser->clear();
      int c;
      char *p;
      for(c=0;c<playlist[ch].length;c++) {
	p = playlist[ch].song(c);
	while(*p!='\0') p++;
	while(*p!='/') p--;
	gui->chan[ch]->playlist_browser->add(p+1,NULL);
      }
    }
  }
  synchronous.reply();
  return(res);
}

void Stream_mixer::upd_timeline(int ch, float res) {
  char temp[10];
  bool changed = false;
  
  if((res - xach[ch]->time.f)>0.01) {
    xach[ch]->time.f = res;
    changed = true;
  }
  
  if(changed)
    gui->chan[ch]->timeline_slider->value(xach[ch]->time.f);
      
  sprintf(temp,"%02u:%02u:%02u",xach[ch]->time.h,xach[ch]->time.m,xach[ch]->time.s);
  gui->chan[ch]->time->value(temp);

}

void Stream_mixer::updchan(int ch, float res) {

  if(res==3.0) {
    gui->status_main->value("urg... syncing channel");
  } else if(res==2.0) {
    switch(xach[ch]->playmode) {
    case 0: /* LOOP */
      gui->chan[ch]->timeline_slider->value(0.0);
      gui->chan[ch]->time->value("00:00:00");
      xach[ch]->stop();
      xach[ch]->play();
      break;
    case 1: /* CONT */
      int newsong;
      newsong = gui->chan[ch]->playlist_browser->value()+1;
      gui->chan[ch]->timeline_slider->value(0.0);
      gui->chan[ch]->time->value("00:00:00");
      
      xach[ch]->set(playlist[ch].selected->next->name);
      if(xach[ch]->opened) {
	playlist[ch].select(newsong-1);
	gui->chan[ch]->name->value(gui->chan[ch]->playlist_browser->text(newsong));
	gui->chan[ch]->playlist_browser->value(newsong);
	xach[ch]->play();
      } else {
	if(have_gui)
	  gui->status_main->value("Doom damnit! can't open the file selected");
	fprintf(stderr,"error opening file on channel %u\n",ch);
	xach[ch]->stop();
      }
      break;
    case 2: /* PLAY */
      gui->chan[ch]->timeline_slider->value(0.0);
      gui->chan[ch]->time->value("00:00:00");
      xach[ch]->stop();
      break;
    }
  } else {
    if(have_gui)
      upd_timeline(ch, res);
  }
}

==> MuSE-0.5.1.tar.gz: MuSE-0.5.1/jmixer.cpp <==
/* jmixer.cpp - part of MUSE
 * (c)2000 Denis Roio aka jaromil
 * c++ engine for multiple mp3 realtime stream mixing
 */

#include <iostream.h>
#include <stdio.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/soundcard.h>
#include <string.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <termios.h>
#include <sys/time.h>
#include <config.h>

#include "jmixer.h"
#include "audioproc.h"

Stream_mixer::Stream_mixer() {

  /* well... let's initialize channels with mp3 decoders */
  int i;
  for(i=0;i<MAX_CHANNELS;i++)
    xach[i] = new XaudioChannel();
  
  dsp = 0;
  have_gui = false;

  dspout = false;
  icecastout = false;
  fileout = false;
  quit = false;

  /* Initialization as a synchronous object */
  synchronous.registerObjectName("Stream_mixer");

}

void Stream_mixer::run() {

  if(debug)
    cerr << selfName() << " running on thread " << (int)pthread_self() << "\n";

  while(!quit) {
    
    synchronous.select.accept(0, id_stream_mixer);

    synchronous.select.waitUntil(1, LTI::Kernel::now()+50);
    synchronous.wait();

    cafudda();
  }

  cerr << selfName() << " exiting " << "\n";

}

Stream_mixer::~Stream_mixer() {
  int i;

  if(dsp!=0)
    close_soundcard();
  
  if(icecastout)
    stop_icecast();

  for(i=0;i<MAX_CHANNELS;i++)
    delete xach[i];

}

void Stream_mixer::register_gui(Amuse *reg_gui) { 
  gui = reg_gui; 
  have_gui = true;
  if(HAVE_LAME) {
    char temp[256];
    lame.version();
    sprintf(temp,"Encoding with Lame %s",lame.ver);
    gui->status_lame->value(temp);
  }
  int c;
  for(c=0;c<MAX_CHANNELS;c++) {
    if(playlist[c].loaded) {
      int cc;
      char *p;
      for(cc=0;cc<playlist[c].length;cc++) {
	p = playlist[c].song(cc);
	while(*p!='\0') p++;
	while(*p!='/') p--;
	gui->chan[c]->playlist_browser->add(p+1,NULL);
      }
    }
  }
}

bool Stream_mixer::open_soundcard()
{
    int format,tstereo,speed,caps;
    static int val;
    
    if((dsp=open("/dev/dsp",O_RDWR))==-1) {
        perror("Error opening /dev/dsp: ");
	return(false);
    } else
      fprintf(stderr,"Found soundcard on /dev/dsp\n");

    /* BUFFER FRAGMENTATION */
    val = (FRAGCOUNT << 16) | FRAGSIZE;

    if(ioctl(dsp,SNDCTL_DSP_SETFRAGMENT,&val)==-1)
      fprintf(stderr,"failed to set dsp buffers\n");

    /* or LE */
    format = AFMT_S16_NE;

    tstereo = 1; /* only stereo _DSP_ in/out */
    speed = SAMPLE_RATE; /* 44100hz mixing */

    ioctl(dsp, SNDCTL_DSP_GETCAPS, &caps);
    if(caps & DSP_CAP_DUPLEX) {
      fullduplex = true;
      fprintf(stderr,"full duplex supported. good\n");
      ioctl(dsp, SNDCTL_DSP_SETDUPLEX, 0);
      ioctl(dsp, DSP_CAP_DUPLEX, 0);
    } else {
      fprintf(stderr,"full duplex is not supported\n");
      fullduplex = false;
    }

    /* FORMAT */
    if(ioctl(dsp,SNDCTL_DSP_SETFMT,&format)==-1)
      fprintf(stderr,"failed to set data format\n");

    /* CHANNELS */
    if(ioctl(dsp,SNDCTL_DSP_STEREO,&tstereo)==-1)
      fprintf(stderr,"something went wrong with the stereo setting\n");

    /* SAMPLERATE */
    if(ioctl(dsp,SNDCTL_DSP_SPEED,&speed)==-1)
      fprintf(stderr,"speed setting failed\n");

    /* GET FRAG SIZE BACK */
    if(ioctl(dsp,SNDCTL_DSP_GETBLKSIZE,&val)==-1)
      fprintf(stderr,"get block size failed to return\n");

    fprintf(stderr,"mixing 16bit %dHz stereo\n",speed);

    dspout = false;

    livein = new LiveIn(speed, tstereo+1, &dsp);

    return(true);
} /* open_soundcard */

bool Stream_mixer::set_icecast(char *ip, int port, char *pass, char *mountpoint, int bps, char *name, char *url, char *desc, int login) {

  if(!ice.set_server(ip,port,pass,mountpoint,bps,login)) {
    if(have_gui)
      gui->status_main->value("doom damnit! i can't resolve icecast hostname");
    else
      fprintf(stderr,"Can't resolve icecast server hostname\n");
    synchronous.reply();
    return(false);
  }

  ice.set_meta(name, url, desc);
      
  if((!lame.initialized)||(lame.bps!=bps)) {
    gui->status_main->value("ouch! lame is not initialized!");
    synchronous.reply();
    return(false);
  }
  
  if(ice.start()) { /* we are connected */
    icecastout = true;

    /* notice the user */
    if(have_gui) {
      char temp[256];
      gui->status_main->value("WE GOT MILK !!");
      sprintf(temp,"STREAMING ON [ http://%s:%u/%s ]",ip,port,mountpoint);
      gui->status_ice->value(temp);
    } else
      fprintf(stderr,"Estabilished %ubps stream on %s:%u/%s\n",bps,ip,port,mountpoint);

  } else { /* connection failed */
    
    icecastout = false;
    if(have_gui) {
      char temp[256];
      sprintf(temp,"Got problems streaming on server %s. No milk??",ip);
      gui->status_main->value(temp);
    } else
      fprintf(stderr,"Got problems streaming on server %s. No milk??\n",ip);
  }
  
  synchronous.reply();
  return(icecastout);
}

void Stream_mixer::set_lame(int bps, int freq, int mode, int quality, int lowpass, int highpass) {
  lame.init(bps, freq, mode, quality, lowpass, highpass);
  synchronous.reply();
}

void Stream_mixer::stop_icecast() {
  ice.stop();
  icecastout = false;

  if (have_gui) {
    gui->status_main->value("Stopped streaming to icecast server");
    gui->status_ice->value("");
  } else
    fprintf(stderr,"Stopped streaming to icecast server\n");

  synchronous.reply();
}

int Stream_mixer::set_mp3file(char *file, int bps) {
  int res;
  
  res = mp3file.openmp3(file);
  if(res>0) fileout = true;

  /*  
      if((!lame.initialized)||(lame.bps!=bps))
      lame.init(bps);
  */
  
  synchronous.reply();
  return(res);
}

void Stream_mixer::stop_mp3file() {
  mp3file.closemp3();
  fileout = false;
  synchronous.reply();
}

void Stream_mixer::cafudda()
{
  int i,j;
  //  int num = 0;
  float res;
  int max = 0;

  /* j is the number of 16bit samples (not counting stereo inrterleaving)*/
  j=BUF_SIZE>>1;
  
  memset(process_buffer,0,BUF_SIZE<<2); /* assuming sizeof(int)==4 */

  have_sound = false;

  for(i=0;i<MAX_CHANNELS;i++) {

    if(xach[i]->on) {
      res = xach[i]->mix(process_buffer);
      updchan(i,res);
      if(max < xach[i]->in_smp) max = xach[i]->in_smp;

      /* notice that we have pcm to process (at least 1 chan) */
      have_sound = true;
      
    } /* if(xach[i].on) */
  } /* for(i=0;i<MAX_CHANNELS;i++) */
  
  if(livein->on) {
    int ires;
    ires = livein->mix(process_buffer);
    if(max < ires) max = ires;
    
    /* notice that we have pcm to process (at least 1 chan) */
    have_sound = true;
  }
  
  if(have_sound) {
    
    /* CLIPPING */
    clip_audio(process_buffer,audio_buffer,max);
    
    /* ENCODE AUDIO */
    if(fileout||icecastout)
      lame.encode(&audio_buffer[0],max>>2);
    
    /* WRITE 2 MP3 FILE */
    if(fileout)
      mp3file.writemp3();
    
    /* SEND 2 ICECAST */
    if(icecastout) {
      if(ice.send()!=0) {
	if(have_gui)
	  gui->status_main->value("Send error: cannot send stream to icecast server");
	else
	  fprintf(stderr,"Send error: cannot send stream to icecast server\n");
      }
    }

    /* WRITE 2 DSP */
    if(dspout)
      write(dsp,audio_buffer,max);

  }

  /* compute and draw levels */
  if(have_gui)
    gui->mix_monitor->draw_levels(get_level(audio_buffer,max));
     
}

void Stream_mixer::pause_channel(int ch) {

  if(xach[ch]->opened)
    xach[ch]->on = !xach[ch]->on;
  
  synchronous.reply();
}

bool Stream_mixer::set_channel(int ch, int playlist_pos) {

  bool res;
  char *sel = playlist[ch].song(playlist_pos);

  if(sel==NULL)
    return(false);


  if(strstr(sel,".ogg")!=NULL) {
#ifdef HAVE_VORBIS
    if(xach[ch]->type != OGGCHAN) {
      delete xach[ch];
      xach[ch] = new OggChannel();
    }
#else
    fprintf(stderr,"can't open .ogg files, OggVorbis support not compiled in\n");
    synchronous.reply();
    return(false);
#endif
  } else if(strstr(sel,".mp3")!=NULL) {
    if(xach[ch]->type != MP3CHAN) {
      delete xach[ch];
      xach[ch] = new XaudioChannel();
    }
  }
  
  res = xach[ch]->set(playlist[ch].song(playlist_pos));

  if(!res) {
    if(have_gui) {
      gui->status_main->value("Doom damnit! can't open the file selected");
    } else {
      fprintf(stderr,"error opening file %s\n", playlist[ch].song(playlist_pos));
    }
  } else {
    playlist[ch].select(playlist_pos);
    if(have_gui) {
      char temp[30];
      sprintf(temp,"File loaded on channel [%u]",ch);
      gui->status_main->value(temp);
    }
  }

  synchronous.reply();
  return(res);
}

void Stream_mixer::set_all_volumes(float *vol) {
  int c;
  for(c=0;c<MAX_CHANNELS;c++) {
    xach[c]->volume = vol[c];
  }
  synchronous.reply();
}

void Stream_mixer::set_volume(int ch, float vol) {

  xach[ch]->volume = vol;

  synchronous.reply();
}

void Stream_mixer::set_speed(int ch, int speed) {

  xach[ch]->speed = speed;

  synchronous.reply();
}

void Stream_mixer::play_channel(int ch) {
  
  if(xach[ch]->opened) {
    xach[ch]->play();
  }

  synchronous.reply();
}
  
bool Stream_mixer::stop_channel(int ch) {

  bool res = true;

  res = xach[ch]->stop();

  /* reset the speed when stopping */
  xach[ch]->speed = 100;
  gui->chan[ch]->speed_slider->value(100);

  synchronous.reply();
  return(res);

}

bool Stream_mixer::set_position(float pos, int ch) {
  bool res = true;

  res = xach[ch]->pos(pos);

  if(!res) {
    char temp[50];
    sprintf(temp,"Error in seeking file on channel [%u]",ch);      
    if(have_gui)
      gui->status_main->value(temp);
    else
      fprintf(stderr,"%s\n",temp);
  }

  synchronous.reply();
  return(res);
}

bool Stream_mixer::set_live(bool stat) {
  if(!((dspout)&&(!fullduplex)&&(stat)))
    livein->on = stat;

  synchronous.reply();
  return(livein->on&stat);
}

bool Stream_mixer::set_lineout(bool stat) {
  if(!((livein->on)&&(!fullduplex)&&(stat)))
    dspout = stat;

  synchronous.reply();
  return(dspout&stat);
}

void Stream_mixer::close_soundcard() {
  ioctl(dsp, SNDCTL_DSP_RESET, 0);
  close(dsp);
}

bool Stream_mixer::set_playlist(char *file, int ch) {
  bool res = false;
  res = playlist[ch].set(file);
  if(res) {
    if(have_gui) {
      gui->chan[ch]->playlist_browser->clear();
      int c;
      char *p;
      for(c=0;c<playlist[ch].length;c++) {
	p = playlist[ch].song(c);
	while(*p!='\0') p++;
	while(*p!='/') p--;
	gui->chan[ch]->playlist_browser->add(p+1,NULL);
      }
    }
  }
  synchronous.reply();
  return(res);
}

void Stream_mixer::upd_timeline(int ch, float res) {
  char temp[10];
  bool changed = false;
  
  if((res - xach[ch]->time.f)>0.01) {
    xach[ch]->time.f = res;
    changed = true;
  }
  
  if(changed)
    gui->chan[ch]->timeline_slider->value(xach[ch]->time.f);
      
  sprintf(temp,"%02u:%02u:%02u",xach[ch]->time.h,xach[ch]->time.m,xach[ch]->time.s);
  gui->chan[ch]->time->value(temp);

}

void Stream_mixer::updchan(int ch, float res) {

  if(res==3.0) {
    gui->status_main->value("urg... syncing channel");
  } else if(res==2.0) {
    switch(xach[ch]->playmode) {
    case 0: /* LOOP */
      gui->chan[ch]->timeline_slider->value(0.0);
      gui->chan[ch]->time->value("00:00:00");
      xach[ch]->stop();
      xach[ch]->play();
      break;
    case 1: /* CONT */
      int newsong;
      xach[ch]->stop();
      newsong = gui->chan[ch]->playlist_browser->value();
      gui->chan[ch]->timeline_slider->value(0.0);
      gui->chan[ch]->time->value("00:00:00");

      do {
	newsong++;
	
	/* if last song in playlist */
	if((newsong-1)>playlist[ch].length) break;
	
	/* -1 'cause fltkbrowser starts from 1 */
	playlist[ch].select(newsong-1);
      } while(!xach[ch]->set(playlist[ch].selected->name));
	
      if(xach[ch]->opened) {
	
	gui->chan[ch]->name->value(gui->chan[ch]->playlist_browser->text(newsong));
	gui->chan[ch]->playlist_browser->value(newsong);
	xach[ch]->play();
      } else 
	gui->chan[ch]->name->value("");

      break;
    case 2: /* PLAY */
      gui->chan[ch]->timeline_slider->value(0.0);
      gui->chan[ch]->time->value("00:00:00");
      xach[ch]->stop();
      break;
    }
  } else {
    if(have_gui)
      upd_timeline(ch, res);
  }
}
